#include "modechange.hpp"

ModeChangeData::ModeChangeData() : cur_mode(0), old_mode(0), 
                                   numOldTasks(0), numSuspended(0)
{
    pthread_mutex_init(&mutex, 0);
    pthread_cond_init(&bmanager, 0); 

    for (unsigned i=0; i<NTASKS; i++) 
        pthread_cond_init(&tcond[i], 0);
    
    for (unsigned i=0; i<NTASKS; i++)
        for (unsigned j=0; j<NMODES; j++) 
            array[i][j] = false;
}


void ModeChangeData::setTaskModes(int i, int m)
{
    array[i][m] = true;
}

bool ModeChangeData::checkMode(int i)
{
    bool r = true;
    pthread_mutex_lock(&mutex);
    if (!array[i][cur_mode]) {
        numSuspended++;
        if (numSuspended == numOldTasks) pthread_cond_signal(&bmanager);
        pthread_cond_wait(&tcond[i], &mutex);
        r = false;
    } 
    pthread_mutex_unlock(&mutex);
    return r;
}

void ModeChangeData::change(int n)
{
    if (n == cur_mode) return;

    pthread_mutex_lock(&mutex);
    old_mode = cur_mode;
    cur_mode = n;
    numOldTasks = 0;
    for (unsigned i=0; i<NTASKS; i++) 
        if (array[i][old_mode] && !array[i][cur_mode]) numOldTasks++;
    
    numSuspended = 0;

    pthread_cond_wait(&bmanager, &mutex);

    pthread_mutex_unlock(&mutex);
}

void ModeChangeData::activateAllTasks()
{
    pthread_mutex_lock(&mutex);

    for (int i=0; i<NTASKS; i++) 
        if (!array[i][old_mode] && array[i][cur_mode])
            pthread_cond_signal(&tcond[i]);

    pthread_mutex_unlock(&mutex);
}

