#include "rttask.hpp"
#include <pthread.h>
#include "time_utils.hpp"
#include <cstdlib>
#include <iostream>

using namespace std;

void *thread_code(void *arg) 
{
    PeriodicTask *pt = (PeriodicTask *) arg;

    pt->run();
    return NULL;
}

PeriodicTask::PeriodicTask(int i, int prio, long p, int ws, ModeChangeData &mc) : index(i), priority(prio),
                                                                                  period_us(p), 
                                                                                  wcet_sim(ws), 
                                                                                  ndlinemiss(0),
                                                                                  mcObj(mc)

{
}

pthread_t PeriodicTask::start()
{
    pthread_t tid;
    sched_param param;
    pthread_attr_t a;

    pthread_attr_init(&a);

    pthread_attr_setschedpolicy(&a, SCHED_FIFO);
    param.sched_priority = priority;
    pthread_attr_setschedparam(&a, &param);
    pthread_create(&tid, &a, thread_code, this);
    
    pthread_attr_destroy(&a);

    return tid;
}

void PeriodicTask::run()
{
  struct timespec next;
  struct timespec now;
  int dline_miss_count = 0;
  int a = 13, b = 23;

  clock_gettime(CLOCK_REALTIME, &next);
  while (1) {
      if (lmsg != "") 
          cout << "Task " << index << ": " << lmsg << endl;

      for (int j=0; j < wcet_sim; j++) a *= b;

      clock_gettime(CLOCK_REALTIME, &now);
      timespec_add_us(&next, period_us);

      if (timespec_cmp(&now, &next) > 0) {
          cerr << "Deadline miss for thread " << index << endl;
          cerr << "now: " << now.tv_sec << " sec " << now.tv_nsec << " nsec     next: " 
               <<  next.tv_sec << " sec " << next.tv_nsec << " nsec" << endl; 
          dline_miss_count ++;
          if (dline_miss_count > ndlinemiss) exit(-1);
      }
      else dline_miss_count = 0;
      
      if (mcObj.checkMode(index)) 
          clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME,
                          &next, NULL);
      else 
          clock_gettime(CLOCK_REALTIME, &next);
      
  }
}


void PeriodicTask::log(const std::string &s)
{
    lmsg = s;
}

// void *thread_code(void *arg) {
//   struct periodic_data_struct *ps = (struct periodic_data_struct *) arg;
//   int j;
//   int a = 13, b = 17; 
//   int i=0;
//   int dline_miss_count = 0;

//   struct timespec next;
//   struct timespec now;
//   sched_param param;

//   clock_gettime(CLOCK_REALTIME, &next);
//   while (1) {
//       //i++;
//     clock_gettime(CLOCK_REALTIME, &now);
//     timespec_add_us(&next, ps->period_us);

//     fprintf(stderr,"th: %d, now is %ld, waking up at %ld\n", 
// 	   ps->index, now.tv_nsec, next.tv_nsec);

//     if (timespec_cmp(&now, &next) > 0) {
//       fprintf(stderr, "Deadline miss for thread %d\n", ps->index);
//       fprintf(stderr, "now: %ld sec %ld nsec     next: %ld sec %ldnsec \n", 
// 	      now.tv_sec, now.tv_nsec, next.tv_sec, next.tv_nsec);
//       dline_miss_count ++;
//       if (dline_miss_count > ps->ndlinemiss) exit(-1);
//     }
//     else dline_miss_count = 0;

//     clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME,
// 		    &next, NULL);

//     fprintf(stderr,"th: %d started\n", ps->index);

//     for (j=0; j<ps->wcet_sim/2; j++) a *= b;
    
//     param.priority = ps->highp;
//     sched_setparam(pthread_self(), &param);

//     for (j=0; j<ps->wcet_sim/2; j++) a *= b;

//     param.priority = ps->lowp;
//     sched_setparam(pthread_self(), &param);

//     fprintf(stderr,"th: %d completed\n", ps->index);
//   }
//   return NULL;
// }
